const WebpackDevServer = require('webpack-dev-server');
const webpack = require('webpack');
const open = require('open');
const appConfig = require('./appConfig');
const getEnvironmentInfo = require('./extendEnv');
const { parseArgsToJson, getProjectPath } = require('./utils');

function openStartRun(webpackFactory) {
  process.env.BABEL_ENV = 'development';
  process.env.NODE_ENV = 'development';

  const PORT = process.env.PORT || parseArgsToJson.port || appConfig.startDefaultPort;
  const HOST = process.env.HOST || '0.0.0.0';

  const extendEnv = getEnvironmentInfo();

  const options = {
    allowedHosts: 'all',
    hot: true,
    host: HOST,
    port: PORT,
    compress: true,
    historyApiFallback: {
      // https://github.com/facebook/create-react-app/issues/387.
      disableDotRule: true
    },
    client: {
      overlay: false,
      progress: false
    },
    static: {
      directory: getProjectPath(appConfig.startOutput.path),
      publicPath: appConfig.startOutput.publicPath || '/',
      watch: {
        aggregateTimeout: 300,
        poll: 1000,
        ignored: /node_modules/
      }
    }
  };

  const startOpenPath = appConfig.startOpenPath ? appConfig.startOpenPath : '';

  try {
    const config = webpackFactory({ ...appConfig, port: PORT, startOpenPath }, extendEnv);
    const compiler = webpack(config);
    const server = new WebpackDevServer(options, compiler);
    let hasOpenBrowser = false;

    // 添加钩子函数，弹出浏览器
    compiler.hooks.done.tap('OpenBrowser', () => {
      if (!hasOpenBrowser) {
        hasOpenBrowser = true;
        setTimeout(() => {
          open(`http://localhost:${PORT}${startOpenPath}`);
        }, 600);
      }
    });

    server.startCallback(err => {
      if (err) {
        // eslint-disable-next-line no-console
        console.log(err);
      }
    });
    ['SIGINT', 'SIGTERM'].forEach(sig => {
      process.on(sig, () => {
        server.close();
        process.exit();
      });
    });
  } catch (err) {
    if (err && err.message) {
      // eslint-disable-next-line no-console
      console.log(err.message);
    }
    process.exit(1);
  }
}

module.exports = openStartRun;
