const { merge } = require('webpack-merge');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const { getProjectPath } = require('../common/utils');
const webpackCommon = require('../common/webpack.common');

module.exports = (isProd, appConfig, extendEnv) => {
  const result = {
    plugins: [
      new HtmlWebpackPlugin({
        inject: true,
        template: getProjectPath(appConfig.htmlPluginTemplate),
        templateParameters: isProd
          ? (compilation, assets, tags, options) => {
              tags.headTags.forEach(tag => {
                if (tag.tagName === 'script') {
                  tag.attributes.async = true;
                }
              });
              return {
                htmlWebpackPlugin: { options }
              };
            }
          : null
      })
    ]
  };

  if (isProd) {
    result.plugins.push(
      new MiniCssExtractPlugin({
        filename: `css/[name].css?v=${extendEnv.version}`
      })
    );
  }

  return merge(webpackCommon(appConfig, extendEnv, { isProd, isServer: false }), result);
};
